@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2009 Red Hat, Inc.
+ * (C) Copyright 2007 - 2010 Red Hat, Inc.
  */
 
 #ifndef EAP_METHOD_TLS_H
@@ -25,16 +25,11 @@
 
 #include "wireless-security.h"
 
-typedef struct {
-	struct _EAPMethod parent;
+typedef struct _EAPMethodTLS EAPMethodTLS;
 
-	gboolean phase2;
-} EAPMethodTLS;
-
-EAPMethodTLS * eap_method_tls_new (const char *glade_file,
-                                   WirelessSecurity *parent,
-                                   NMConnection *connection,
-                                   gboolean phase2);
+EAPMethodTLS *eap_method_tls_new (WirelessSecurity *ws_parent,
+                                  NMConnection *connection,
+                                  gboolean phase2);
 
 #endif /* EAP_METHOD_TLS_H */
 
