@@ -17,11 +17,12 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 Red Hat, Inc.
+ * (C) Copyright 2007 - 2010 Red Hat, Inc.
  */
 
+#include "config.h"
+
 #include <glib/gi18n.h>
-#include <glade/glade.h>
 #include <ctype.h>
 #include <string.h>
 
@@ -30,11 +31,18 @@
 
 #include "eap-method.h"
 #include "wireless-security.h"
-#include "gconf-helpers.h"
 
 #define I_NAME_COLUMN   0
 #define I_METHOD_COLUMN 1
 
+struct _EAPMethodTTLS {
+	EAPMethod parent;
+
+	GtkSizeGroup *size_group;
+	WirelessSecurity *sec_parent;
+	gboolean is_editor;
+};
+
 static void
 destroy (EAPMethod *parent)
 {
@@ -42,7 +50,6 @@ destroy (EAPMethod *parent)
 
 	if (method->size_group)
 		g_object_unref (method->size_group);
-	g_slice_free (EAPMethodTTLS, method);
 }
 
 static gboolean
@@ -54,10 +61,10 @@ validate (EAPMethod *parent)
 	EAPMethod *eap = NULL;
 	gboolean valid = FALSE;
 
-	if (!eap_method_validate_filepicker (parent->xml, "eap_ttls_ca_cert_button", TYPE_CA_CERT, NULL, NULL))
+	if (!eap_method_validate_filepicker (parent->builder, "eap_ttls_ca_cert_button", TYPE_CA_CERT, NULL, NULL))
 		return FALSE;
 
-	widget = glade_xml_get_widget (parent->xml, "eap_ttls_inner_auth_combo");
+	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_inner_auth_combo"));
 	g_assert (widget);
 
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
@@ -82,19 +89,19 @@ add_to_size_group (EAPMethod *parent, GtkSizeGroup *group)
 		g_object_unref (method->size_group);
 	method->size_group = g_object_ref (group);
 
-	widget = glade_xml_get_widget (parent->xml, "eap_ttls_anon_identity_label");
+	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_anon_identity_label"));
 	g_assert (widget);
 	gtk_size_group_add_widget (group, widget);
 
-	widget = glade_xml_get_widget (parent->xml, "eap_ttls_ca_cert_label");
+	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_ca_cert_label"));
 	g_assert (widget);
 	gtk_size_group_add_widget (group, widget);
 
-	widget = glade_xml_get_widget (parent->xml, "eap_ttls_inner_auth_label");
+	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_inner_auth_label"));
 	g_assert (widget);
 	gtk_size_group_add_widget (group, widget);
 
-	widget = glade_xml_get_widget (parent->xml, "eap_ttls_inner_auth_combo");
+	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_inner_auth_combo"));
 	g_assert (widget);
 
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
@@ -127,13 +134,13 @@ fill_connection (EAPMethod *parent, NMConnection *connection)
 
 	nm_setting_802_1x_add_eap_method (s_8021x, "ttls");
 
-	widget = glade_xml_get_widget (parent->xml, "eap_ttls_anon_identity_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_anon_identity_entry"));
 	g_assert (widget);
 	text = gtk_entry_get_text (GTK_ENTRY (widget));
 	if (text && strlen (text))
 		g_object_set (s_8021x, NM_SETTING_802_1X_ANONYMOUS_IDENTITY, text, NULL);
 
-	widget = glade_xml_get_widget (parent->xml, "eap_ttls_ca_cert_button");
+	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_ca_cert_button"));
 	g_assert (widget);
 	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (widget));
 	if (!nm_setting_802_1x_set_ca_cert (s_8021x, filename, NM_SETTING_802_1X_CK_SCHEME_PATH, &format, &error)) {
@@ -141,11 +148,7 @@ fill_connection (EAPMethod *parent, NMConnection *connection)
 		g_clear_error (&error);
 	}
 
-	nm_gconf_set_ignore_ca_cert (nm_setting_connection_get_uuid (s_con),
-	                             FALSE,
-	                             eap_method_get_ignore_ca_cert (parent));
-
-	widget = glade_xml_get_widget (parent->xml, "eap_ttls_inner_auth_combo");
+	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_inner_auth_combo"));
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
 	gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter);
 	gtk_tree_model_get (model, &iter, I_METHOD_COLUMN, &eap, -1);
@@ -167,13 +170,14 @@ inner_auth_combo_changed_cb (GtkWidget *combo, gpointer user_data)
 	GtkTreeIter iter;
 	GtkWidget *eap_widget;
 
-	vbox = glade_xml_get_widget (parent->xml, "eap_ttls_inner_auth_vbox");
+	vbox = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_inner_auth_vbox"));
 	g_assert (vbox);
 
 	/* Remove any previous wireless security widgets */
 	children = gtk_container_get_children (GTK_CONTAINER (vbox));
 	for (elt = children; elt; elt = g_list_next (elt))
 		gtk_container_remove (GTK_CONTAINER (vbox), GTK_WIDGET (elt->data));
+	g_list_free (children);
 
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
 	gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter);
@@ -182,6 +186,7 @@ inner_auth_combo_changed_cb (GtkWidget *combo, gpointer user_data)
 
 	eap_widget = eap_method_get_widget (eap);
 	g_assert (eap_widget);
+	gtk_widget_unparent (eap_widget);
 
 	if (method->size_group)
 		eap_method_add_to_size_group (eap, method->size_group);
@@ -194,11 +199,10 @@ inner_auth_combo_changed_cb (GtkWidget *combo, gpointer user_data)
 
 static GtkWidget *
 inner_auth_combo_init (EAPMethodTTLS *method,
-                       const char *glade_file,
                        NMConnection *connection,
                        NMSetting8021x *s_8021x)
 {
-	GladeXML *xml = EAP_METHOD (method)->xml;
+	EAPMethod *parent = (EAPMethod *) method;
 	GtkWidget *combo;
 	GtkListStore *auth_model;
 	GtkTreeIter iter;
@@ -218,10 +222,10 @@ inner_auth_combo_init (EAPMethodTTLS *method,
 			phase2_auth = nm_setting_802_1x_get_phase2_autheap (s_8021x);
 	}
 
-	em_pap = eap_method_simple_new (glade_file,
-	                                method->sec_parent,
+	em_pap = eap_method_simple_new (method->sec_parent,
 	                                connection,
 	                                EAP_METHOD_SIMPLE_TYPE_PAP,
+	                                TRUE,
 	                                method->is_editor);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
@@ -234,10 +238,10 @@ inner_auth_combo_init (EAPMethodTTLS *method,
 	if (phase2_auth && !strcasecmp (phase2_auth, "pap"))
 		active = 0;
 
-	em_mschap = eap_method_simple_new (glade_file,
-	                                   method->sec_parent,
+	em_mschap = eap_method_simple_new (method->sec_parent,
 	                                   connection,
 	                                   EAP_METHOD_SIMPLE_TYPE_MSCHAP,
+	                                   TRUE,
 	                                   method->is_editor);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
@@ -250,10 +254,10 @@ inner_auth_combo_init (EAPMethodTTLS *method,
 	if (phase2_auth && !strcasecmp (phase2_auth, "mschap"))
 		active = 1;
 
-	em_mschap_v2 = eap_method_simple_new (glade_file,
-	                                      method->sec_parent,
+	em_mschap_v2 = eap_method_simple_new (method->sec_parent,
 	                                      connection,
 	                                      EAP_METHOD_SIMPLE_TYPE_MSCHAP_V2,
+	                                      TRUE,
 	                                      method->is_editor);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
@@ -266,10 +270,10 @@ inner_auth_combo_init (EAPMethodTTLS *method,
 	if (phase2_auth && !strcasecmp (phase2_auth, "mschapv2"))
 		active = 2;
 
-	em_chap = eap_method_simple_new (glade_file,
-	                                 method->sec_parent,
+	em_chap = eap_method_simple_new (method->sec_parent,
 	                                 connection,
 	                                 EAP_METHOD_SIMPLE_TYPE_CHAP,
+	                                 TRUE,
 	                                 method->is_editor);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
@@ -282,7 +286,7 @@ inner_auth_combo_init (EAPMethodTTLS *method,
 	if (phase2_auth && !strcasecmp (phase2_auth, "chap"))
 		active = 3;
 
-	combo = glade_xml_get_widget (xml, "eap_ttls_inner_auth_combo");
+	combo = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_inner_auth_combo"));
 	g_assert (combo);
 
 	gtk_combo_box_set_model (GTK_COMBO_BOX (combo), GTK_TREE_MODEL (auth_model));
@@ -305,67 +309,47 @@ update_secrets (EAPMethod *parent, NMConnection *connection)
 }
 
 EAPMethodTTLS *
-eap_method_ttls_new (const char *glade_file,
-                     WirelessSecurity *parent,
+eap_method_ttls_new (WirelessSecurity *ws_parent,
                      NMConnection *connection,
                      gboolean is_editor)
 {
+	EAPMethod *parent;
 	EAPMethodTTLS *method;
 	GtkWidget *widget;
-	GladeXML *xml;
 	GtkFileFilter *filter;
 	NMSetting8021x *s_8021x = NULL;
 	const char *filename;
 
-	g_return_val_if_fail (glade_file != NULL, NULL);
-
-	xml = glade_xml_new (glade_file, "eap_ttls_notebook", NULL);
-	if (xml == NULL) {
-		g_warning ("Couldn't get eap_ttls_widget from glade xml");
+	parent = eap_method_init (sizeof (EAPMethodTTLS),
+	                          validate,
+	                          add_to_size_group,
+	                          fill_connection,
+	                          update_secrets,
+	                          destroy,
+	                          UIDIR "/eap-method-ttls.ui",
+	                          "eap_ttls_notebook",
+	                          "eap_ttls_anon_identity_entry",
+	                          FALSE);
+	if (!parent)
 		return NULL;
-	}
 
-	widget = glade_xml_get_widget (xml, "eap_ttls_notebook");
-	g_assert (widget);
-	g_object_ref_sink (widget);
+	eap_method_nag_init (parent, "eap_ttls_ca_cert_button", connection);
 
-	method = g_slice_new0 (EAPMethodTTLS);
-	if (!method) {
-		g_object_unref (xml);
-		g_object_unref (widget);
-		return NULL;
-	}
-
-	eap_method_init (EAP_METHOD (method),
-	                 validate,
-	                 add_to_size_group,
-	                 fill_connection,
-	                 update_secrets,
-	                 destroy,
-	                 xml,
-	                 widget,
-	                 "eap_ttls_anon_identity_entry");
-
-	eap_method_nag_init (EAP_METHOD (method),
-	                     glade_file,
-	                     "eap_ttls_ca_cert_button",
-	                     connection,
-	                     FALSE);
-
-	method->sec_parent = parent;
+	method = (EAPMethodTTLS *) parent;
+	method->sec_parent = ws_parent;
 	method->is_editor = is_editor;
 
 	if (connection)
 		s_8021x = NM_SETTING_802_1X (nm_connection_get_setting (connection, NM_TYPE_SETTING_802_1X));
 
-	widget = glade_xml_get_widget (xml, "eap_ttls_ca_cert_button");
+	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_ca_cert_button"));
 	g_assert (widget);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (widget), TRUE);
 	gtk_file_chooser_button_set_title (GTK_FILE_CHOOSER_BUTTON (widget),
 	                                   _("Choose a Certificate Authority certificate..."));
 	g_signal_connect (G_OBJECT (widget), "selection-changed",
 	                  (GCallback) wireless_security_changed_cb,
-	                  parent);
+	                  ws_parent);
 	filter = eap_method_default_file_chooser_filter_new (FALSE);
 	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (widget), filter);
 	if (connection && s_8021x) {
@@ -376,14 +360,14 @@ eap_method_ttls_new (const char *glade_file,
 		}
 	}
 
-	widget = glade_xml_get_widget (xml, "eap_ttls_anon_identity_entry");
+	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "eap_ttls_anon_identity_entry"));
 	if (s_8021x && nm_setting_802_1x_get_anonymous_identity (s_8021x))
 		gtk_entry_set_text (GTK_ENTRY (widget), nm_setting_802_1x_get_anonymous_identity (s_8021x));
 	g_signal_connect (G_OBJECT (widget), "changed",
 	                  (GCallback) wireless_security_changed_cb,
-	                  parent);
+	                  ws_parent);
 
-	widget = inner_auth_combo_init (method, glade_file, connection, s_8021x);
+	widget = inner_auth_combo_init (method, connection, s_8021x);
 	inner_auth_combo_changed_cb (widget, (gpointer) method);
 
 	return method;
